/*
 * Copyright (c) 2008, Witold Wysota
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of Witold Wysota may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <QtCore>
#include <QtGui>

class Widget : public QWidget {
  Q_OBJECT
public:
  Widget() : QWidget(){
    QVBoxLayout *lay = new QVBoxLayout(this);
    label = new QLabel(tr("Check if the number below is prime"));
    spinBox = new QSpinBox;
    spinBox->setMinimum(3);
    spinBox->setMaximum(999999999);
    spinBox->setValue(987111113);
    lay->addWidget(label);
    lay->addWidget(spinBox);
    pushButton = new QPushButton;
    pushButton->setCheckable(true);
    pushButton->setText(tr("Run!"));
    connect(pushButton, SIGNAL(toggled(bool)), this, SLOT(run()));
    lay->addWidget(pushButton);
  }
public slots:
  void run(){
    if(!pushButton->isChecked()){
      pushButton->setText(tr("Run!"));
      return;
    }
    pushButton->setText(tr("Abort"));
    int x = spinBox->value();
    bool isPrime = true;
    if(x%2==0)
      isPrime = false;
    int max = qRound(sqrt(x));
    for(int i=3; i<=max && isPrime; i+=2){
      label->setText(tr("Checking %1...").arg(i));
      if( x % i == 0)
        isPrime = false;
      QCoreApplication::processEvents();
      if(!pushButton->isChecked()){
        label->setText(tr("Aborted"));
        return;
      }
    }
    if(isPrime)
      label->setText(tr("%1 is prime").arg(x));
    else
      label->setText(tr("%1 is not prime").arg(x));
    pushButton->setChecked(false);
  }
private:
  QLabel *label;
  QSpinBox *spinBox;
  QPushButton *pushButton;
};

#include "main.moc"

int main(int argc, char **argv){
  QApplication app(argc, argv);
  Widget w;
  w.show();
  return app.exec();
}
